### Name: bioinfer1.JGR
### Title: Dialog box for predicting env. conditions from biological
###   observations
### Aliases: bioinfer1.JGR
### Keywords: regression

### ** Examples

##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--    or do  help(data=index)  for the standard data sets.

## The function is currently defined as
function (coefficientFile, bencntFile = "", runTaxonomic = TRUE, 
    taxonomicFile = "bcntTax.txt", runOTU = TRUE, otuFile = "OTU.txt", 
    mlFile = "MaxLikEnv.txt") 
{
    resultLocation = genResultSpace()
    data(itis.ttable)
    errorMessage = character()
    if (runTaxonomic & !runOTU) {
        localJGRError(paste("If the taxonomic merge is being run, then the", 
            "\noperational taxonomy should be run as well"), 
            resultLocation, geterr = FALSE)
    }
    if (runTaxonomic) {
        bencntMatrix <<- try(read.table(bencntFile, header = TRUE, 
            sep = "\t"))
        if (inherits(bencntMatrix, "try-error")) {
            localJGRError(c(errorMessage, paste("Failure reading file: ", 
                bencntFile, ".", sep = ""), "Check that file is tab-delimited."), 
                resultLocation)
        }
        cat("Starting taxonomic merge.\n")
        cat("If data editor appears, you may attempt to correct spellings\n")
        cat("  by typing them in column 2, or just close the editor to continue\n")
        cat("  without corrections.\n")
        flush.console()
    }
    else {
        taxonomyMatrix <<- try(read.table(taxonomicFile, header = TRUE, 
            sep = "\t"))
        if (inherits(taxonomyMatrix, "try-error")) {
            localJGRError(c(errorMessage, paste("Failure reading taxonomic merge from file:", 
                taxonomicFile), "Check that file exists and is tab-delimited."), 
                resultLocation)
        }
    }
    coefname = try(load(coefficientFile))
    if (inherits(coefname, "try-error")) {
        localJGRError(c(errorMessage, paste("Error importing coefficients file:", 
            coefficientFile), paste("Check that it is an appropriate .rda", 
            "coefficient file.")), resultLocation)
    }
    coefs <<- get(coefname)
    if (!runOTU) {
        otuMatrix <<- try(read.table(otuFile, header = TRUE, 
            sep = "\t"))
        if (inherits(otuMatrix, "try-error")) {
            localJGRError(c(errorMessage, paste("Failure reading operational taxonomic units from", 
                "file:", taxonomicFile), "Check that file exists and is tab-delimited."), 
                resultLocation)
        }
    }
    if (runTaxonomic) {
        taxonomyMatrix <<- get.mismatch(bencntMatrix, itis.ttable)
        if (isS4(taxonomyMatrix)) {
            return("Transfer to Java operation")
        }
        else {
            tmp = try(write.table(taxonomyMatrix, taxonomicFile, 
                sep = "\t"))
            if (inherits(tmp, "try-error")) {
                localJGRError(c(errorMessage, paste("Failure writing to file:", 
                  taxonomicFile)), resultLocation)
            }
            else {
                errorMessage = c(errorMessage, paste("Taxonomy merge completed, written to file:", 
                  taxonomicFile))
            }
        }
    }
    if (runOTU) {
        otuMatrix <<- try(get.otu.jgr(taxonomyMatrix, optlist = coefs$tnames, 
            outputFile = NULL))
        if (inherits(otuMatrix, "try-error")) {
            localJGRError(c(errorMessage, paste("Failure completing generation of operational", 
                "taxonomic units."), paste("Check that file", 
                taxonomicFile, "is formatted", "properly.")), 
                resultLocation)
        }
        tmp = try(write.table(otuMatrix, otuFile, sep = "\t"))
        if (inherits(tmp, "try-error")) {
            localJGRError(c(errorMessage, paste("Error trying to write operational taxonomic units", 
                "to", otuFile)), resultLocation)
        }
        else {
            errorMessage = c(errorMessage, "Generation of operational taxonomic units completed.")
        }
    }
    ssMatrix = try(makess(otuMatrix, plothist = FALSE, prints = FALSE))
    if (inherits(ssMatrix, "try-error")) {
        localJGRError(c(errorMessage, "Failure in generation of site-OTU matrix."), 
            resultLocation)
    }
    mlMatrix = try(mlsolve(ssMatrix, coefs))
    if (inherits(mlMatrix, "try-error")) {
        localJGRError(c(errorMessage, "Failure in generation of maximum likelihood estimates."), 
            resultLocation)
    }
    tmp = try(write.table(mlMatrix, mlFile, sep = "\t", row.names = FALSE))
    if (inherits(tmp, "try-error")) {
        localJGRError(c(errorMessage, paste("Failure writing to file:", 
            mlFile)), resultLocation)
    }
    mlMatrix = as.data.frame(mlMatrix)
    colnames(mlMatrix) = c("Site", "Sediment", "Temperature", 
        "Inconsistent")
    attr(mlMatrix, "title") = "Maximum Likelihood Estimates of Environmental Conditions by Site"
    png(file = file.path(resultLocation, "Abundance Histogram.png"), 
        width = 600, height = 600)
    par(mar = c(4, 4, 3, 1))
    plot(attr(ssMatrix, "histogram"), xlab = "Proportion of abundance", 
        main = "")
    title("Abundance Divided by Total Abundance", line = 2)
    title("for Operational Taxonomic Units", line = 1)
    dev.off()
    tmp = buildresultsXML(object = list(mlMatrix), location = resultLocation, 
        rowNames = FALSE)
    return(mlMatrix)
  }



